%SMART HUS MED REAKTIV KOMPENSERING

%Dette caset omfatter simulering av spenningen i tilknytingspunktet ved
%anvendelse av reaktiv kompensering, alts produksjon og forbruk av 
%reaktiv effekt i perioder hvor spenningsrestriksjoner Vqopp 
%eller Vqned er brutt. Det er ogs mulig at produksjonsenheten 
%kobles ut hvis restriksjoner for utkobling brytes, Vmax og Vmin,
%da m en ny import og spenningsverdi utregnes for de aktuelle periodene.

function [Psol, U_kunde, P_import, Qsol] = Case_3(Unett, Psol, ...
         P_import, Qsol, R, X, a, b, U_kunde, str, Vmax, Vmin, ...
         forbruk_effekt, Vqopp, Vqned)
%Kjrer grunnscenario og anvender spenningsverdiene over dgnet
U_kunde = Case_2(Unett, P_import, Qsol, R, X, a, b, U_kunde, str); 
    
%sjekker om spenningsverdiene er over restriksjoene for reaktiv forbruk
%og om produksjonsenheten er i drift i det aktuelle tidspunktet
for t = 1:str
 if U_kunde(1,t) >= Vqopp && Psol(1,(t-1)*30) > 0
          
%Hvis spenningen er over, og produksjonenheten i drift, s defineres en 
%driftskurve, alts tan(arccos(effektfaktor)) som tilsier forbruk av
%reaktiv effekt tilsvarende Effektfaktor = 0,95
     driftskurven = (-0.32855);
          
%reaktiv effekt regnes ut basert p produksjon og driftskurve
%for de tidsperiodene hvor spenningen er over restriksjonen. Frst regnes
%det ut for halvtime, og deretter oppdateres alle minuttsdataene
    Qsol(1,(t-1)*30)= Psol(1,(t-1)*30) * driftskurven;
    if Qsol(1,(t-2)*30) < 0
        for i = ((t-2)*30):((t-1)*30)
            Qsol(1,i) = Psol(1,(t-1)*30) * driftskurven;
        end
    end
            
%Regner ut ny spenning i tilknytingspunkt, med forbruk av reaktiv effekt i
%de aktuelle periodene hvor restriksjonen er brutt og produksjonenheten er
%i drift. P lik mte som i case 2. 
if t == 1
   a(1,t) 		= ((Unett+sqrt(Unett^2-4*(R*P_import(1,t))+...
                  (X*Qsol(1,t))+(((R*Qsol(1,t)-X*P_import(1,t))...
                  /Unett)^2))))/2;
   b(1,t) 		= (R*Qsol(1,t)-X*P_import(1,t))/Unett;
   U_kunde(1,t)= sqrt ( a(1,t) ^ 2 + b(1,t) ^ 2 );
 else
   a(1,t) 		= ((Unett+sqrt(Unett^2-4*(R*P_import(1,t))+...
                  (X*Qsol(1,(t-1)*30))+(((R*Qsol(1,(t-1)*30)...
                  -X*P_import(1,t))/Unett)^2))))/2;
   b(1,t) 		= (R*Qsol(1,(t-1)*30)-X*P_import(1,t))/Unett;
   U_kunde(1,t) = sqrt ( a(1,t) ^ 2 + b(1,t) ^ 2 );
end

%Hvis spenningen i tilknytingspunktet er strre enn restriksjonen for
%utkobling s kobles produksjonen ut og Psol og Qsol settes til 
%null for de periodene for spenningen er over restriksjonen. 
    if U_kunde(1,t) >= Vmax
      Psol(1,(t-1)*30) = 0;
      Qsol(1,(t-1)*30) = 0;

%Siden Psol og Qsol er oppgitt i minutt, m alle minutt innenfor 
%halvtimen settestil null i de perioder hvor restriksjonen for 
%utkobling er brutt
      if Psol(1,(t-2)*30) == 0
          for i = ((t-2)*30):((t-1)*30)
             Psol(1,i) = 0;
             Qsol(1,i) = 0;
          end
      end

%Hvis lokal produksjon er null, s er import alltid lik forbruk, og 
%derav endres importen.              
             P_import(1,t)   = forbruk_effekt(t);

%Ny spenning i tilknytingspunktet regnes ut nr den lokale
%produksjonsenheten er kobles ut og forbruket forsynes av 
%import fra nettet. P lik mte som i case 2.
        if t == 1
            a(1,t) 		= ((Unett+sqrt(Unett^2-4*(R*P_import(1,t))...
                          +(X*Qsol(1,t))+(((R*Qsol(1,t)-X*...
                          P_import(1,t))/Unett)^2))))/2;
            b(1,t) 		= (R*Qsol(1,t)-X*P_import(1,t))/Unett;
            U_kunde(1,t)= sqrt ( a(1,t) ^ 2 + b(1,t) ^ 2 );
        else
            a(1,t) 		= ((Unett+sqrt(Unett^2-4*(R*P_import(1,t))...
                          +(X*Qsol(1,(t-1)*30))+(((R*Qsol(1,(t-1)*30)...
                          -X*P_import(1,t))/Unett)^2))))/2;
            b(1,t) 		= (R*Qsol(1,(t-1)*30)-X*P_import(1,t))/Unett;
            U_kunde(1,t)= sqrt ( a(1,t) ^ 2 + b(1,t) ^ 2 );
        end
    end

%sjekker om spenningsverdiene er lavere enn restriksjoene for reaktiv
%produksjon og om produksjonsenheten er i drift i det tidspunktet           
    elseif U_kunde(1,t) <= Vqned && t > 2 && Psol(1,(t-1)*30)>0

%Hvis spenningen er under, og produksjonenheten i drift, s defineres en 
%driftskurve, alts tan(arccos(effektfaktor) som tilsier produksjon av
%reaktiv effekt tilsvarende Effektfaktor = 0,95            
        driftskurven = (0.32855);

%reaktiv effekt regnes ut basert p produksjon og driftskurve
%for de tidsperiodene hvor spenningen er lavere enn restriksjonen.
%Frst regnes det ut for halvtime, og deretter oppdateres alle minuttsdata
        Qsol(1,(t-1)*30)= Psol(1,(t-1)*30) * driftskurven;
        if Qsol(1,(t-2)*30) > 0
            for i = ((t-2)*30):((t-1)*30)
                Qsol(1,i) = Psol(1,(t-1)*30) * driftskurven;
            end
        end
            
%Regner ut ny spenning i tilknytingspunkt, 
%med produksjon av reaktiv effekt. P lik mte som i case 2.
        if t == 1
            a(1,t) 		= ((Unett+sqrt(Unett^2-4*(R*P_import(1,t))...
                          +(X*Qsol(1,t))+(((R*Qsol(1,t)-X*...
                          P_import(1,t))/Unett)^2))))/2;
            b(1,t) 		= (R*Qsol(1,t) - X * P_import(1,t) ) / Unett;
            U_kunde(1,t)= sqrt ( a(1,t) ^ 2 + b(1,t) ^ 2 );
        else
            a(1,t) 		= ((Unett+sqrt(Unett^2-4*(R*P_import(1,t))...
                          +(X*Qsol(1,(t-1)*30))+(((R*Qsol(1,(t-1)*30)...
                          -X*P_import(1,t))/Unett)^2))))/2;
            b(1,t) 		= (R*Qsol(1,(t-1)*30)-X*P_import(1,t))/Unett;
            U_kunde(1,t)= sqrt ( a(1,t) ^ 2 + b(1,t) ^ 2 );
        end

%Hvis spenningen i tilknytingspunktet er lavere enn restriksjonen 
%for utkobling s kobles produksjonen ut og Psol og Qsol settes 
%til null for de periodene hvor spenningen er lavere enn restriksjonen. 
        if U_kunde(1,t) <= Vmin && t>2
            Psol(1,(t-1)*30) = 0;
            Qsol(1,(t-1)*30) = 0;

%Siden Psol og Qsol er oppgitt i minutt, m alle minutt innenfor 
%halvtimen settestil null i de perioder hvor restriksjonen for 
%utkobling er brutt
            if Psol(1,(t-2)*30) == 0
                for i = ((t-2)*30):((t-1)*30)
                Psol(1,i) = 0;
                Qsol(1,i) = 0;
                end
            end
        end
        
%Hvis lokal produksjon er null, s er import alltid lik forbruk, og derav
%endres importen. 
         P_import(1,t)   = forbruk_effekt(t);
%ny spenning i tilknytingspunktet utregens for perioder hvor
%produksjonenheten er utkoblet og import av effekt forsyner det lokale
%forbruket
    if t == 1
        a(1,t) 		= ((Unett+sqrt(Unett^2-4*(R*P_import(1,t))...
                      +(X*Qsol(1,t))+(((R*Qsol(1,t)-X*...
                      P_import(1,t))/Unett)^2))))/2;
        b(1,t) 		= (R*Qsol(1,t)-X*P_import(1,t))/Unett;
        U_kunde(1,t)= sqrt ( a(1,t) ^ 2 + b(1,t) ^ 2 );
    else
        a(1,t) 		= ((Unett+sqrt(Unett^2-4*(R*P_import(1,t))...
                      +(X*Qsol(1,(t-1)*30))+(((R*Qsol(1,(t-1)*30)...
                      -X*P_import(1,t))/Unett)^2))))/2;
        b(1,t) 		= (R*Qsol(1,(t-1)*30)-X*P_import(1,t))/Unett;
        U_kunde(1,t)= sqrt ( a(1,t) ^ 2 + b(1,t) ^ 2 );
    end
  end
 end
%returnere maksimale spenning over dgnet
    MaxVerdi = max(U_kunde); 
    MaxVerdi
