%SMART HUS MED REAKTIV KOMPENSERING OG REGULERING AV AKTIV EFFEKT

%Dette caset omfatter simulering av spenningen i tilknytingspunktet ved
%anvendelse av bde kompensering med reaktiv effekt og regulering av aktiv
%effekt innmatning. Aktiv effekt innmatning regulers med  regulere
%forbruket eller lagring til det smarte huset. I denne simuleringen er det
%kun grense for vre spenningsverdi, som vil si at huset ker forbruket
%eller lagrer produksjon for  redusere innmatet effekt. Innmatet effekt
%reduseres til en utregnet verdi hvor spenningen i tilknytningspunktet
%holdes lik grenseverdien i de perioder hvor restriksjonen i utgangspuntket
%ville vrt brutt. 

function [Psol, U_kunde, P_import, Qsol, P_term] = ...
         Case_4(Unett, Psol, P_import, Qsol, R, X, a, b, U_kunde,...
         str, Vmax, Vmin, forbruk_effekt, Vqopp, Vqned, Vlad, P_term)
%Kjrer grunnscenario og anvender spenningsverdiene over dagen
    U_kunde = Case_2(Unett, P_import, Qsol, R, X, a, b, U_kunde, str);
    
%sjekker om spenningsverdiene er over restriksjoene for reaktiv forbruk
%og om produksjonsenheten er i drift i det tidspunktet   
for t = 1:str
    if U_kunde(1,t) >= Vqopp && Psol(1,(t-1)*30) > 0

%Hvis spenningen er over, og produksjonenheten i drift, s defineres en 
%driftskurve, alts tan(arccos(effektfaktor) som tilsier forbruk av
%reaktiv effekt tilsvarende Effektfaktor = 0,95      
        driftskurven             = (-0.32855);

%reaktiv effekt regnes ut basert p produksjon og driftskurve
%for de tidsperiodene hvor spenningen er over restriksjonen. Frst regnes
%det ut for halvtime, og deretter oppdateres alle minuttsdata
    Qsol(1,(t-1)*30)= Psol(1,(t-1)*30) * driftskurven;
    if Qsol(1,(t-2)*30) < 0
        for i = ((t-2)*30):((t-1)*30)
            Qsol(1,i) = Psol(1,(t-1)*30) * driftskurven;
        end
    end
%Regner ut ny spenning i tilknytingspunkt, med forbruk av reaktiv effekt
%Lik mte som i case 2
    if t == 1
        a(1,t) 		= ((Unett+sqrt(Unett^2-4*(R*P_import(1,t))...
                      +(X*Qsol(1,t))+(((R*Qsol(1,t)-X...
                      * P_import(1,t))/Unett)^2))))/2;
        b(1,t) 		= ( R * Qsol(1,t) - X * P_import(1,t))/Unett;
        U_kunde(1,t)= sqrt ( a(1,t) ^ 2 + b(1,t) ^ 2 );
    else
        a(1,t) 		=((Unett+sqrt(Unett^2-4*(R*P_import(1,t))...
                     +(X*Qsol(1,(t-1)*30))+(((R*Qsol(1,(t-1)*30)...
                     -X*P_import(1,t))/Unett)^2))))/2;
        b(1,t) 		=(R*Qsol(1,(t-1)*30)-X*P_import(1,t))/Unett;
        U_kunde(1,t)= sqrt ( a(1,t) ^ 2 + b(1,t) ^ 2 );
    end
    
%I de perioder hvor spenningen er over restriksjonen for regulering av 
%aktiv effekt, alts et kt forbruk eller lagring, s regnes det ut en 
%verdi for forbruk eller lagring. Denne verdien er gitt av produksjonen og 
%forbruket i den aktuelle perioden. Importert effekt endres ved kt 
%forbruk, noe som igjen endrer spenningen i tilknytningspunktet.
%P_import er fortsatt differansen mellom lokal produksjon og forbruk, 
%hvor det kt forbruket/lagringen vil ke forbruket.Importertert effekt er 
%negativ, alts det eksporterteres effekt hvis denne restriksjonen brytes
   if U_kunde(1,t) > Vlad
      Pso               = Psol(1,( t-1 ) * 30);%Produksjon
      Pfor              = forbruk_effekt(t,1); %Forbruk
      Qso               = Qsol(1,( t-1 ) * 30); %Reaktiv effekt 
  
%Det benyttes en Matlab funksjon som regner ut hvor mye 
%forbruket m kes eller lagres slik at P_import resulteter 
%i en spenning i tilknytningspunktet som overholdes restriksjonen.
      problem.objective = @(Plag)sqrt((((Unett+sqrt(Unett^2-4*...
                          (R*(-Pso+Pfor+Plag))+(X*Qso)+...
                          (((R*Qso-X*(-Pso+Pfor+Plag))/Unett)^2))))...
                          /2)^2+((R*Qso-X*(-Pso+Pfor+Plag))...
                          /Unett)^2)-Vlad;
      problem.x0        = 0;
      problem.solver    = 'fzero'; % a required part of the structure
      problem.options   = optimset(@fzero); % default options
      Plag              = fzero(problem);

%Verdier for kt forbruk/lagring gitt av Matlab funksjonen for de ulike 
%tidspunktetene hvor spenningen egentlig hadde vrt hyere
      P_term(1,t)         = Plag;
%Importert effekt har en negativ verdi, denne reduseres mot null nr
%forbruket eller lagringen ker. 
      P_import(1,t)=forbruk_effekt(t)-Psol(1,(t-1)*30)+P_term(1,t);
%En ny spenning regnes ut med den nye importerte effekten. Det er antatt 
%at det ikke er noen restriksjon for hvor mye forbruket kan kes eller 
%hvor mye som kan lagres, noe som vil si at spenningen ndvendigvis 
%vil vre maksimalt lik restriksjonen for kt forbruk/lagring
      a(1,t)   		= ((Unett+sqrt(Unett^2-4*(R*P_import(1,t))...
                      +(X*Qsol(1,(t-1)*30))+(((R*Qsol(1,(t-1)*30)...
                      -X*P_import(1,t))/Unett)^2))))/2;
      b(1,t)   		= (R*Qsol(1,(t-1)*30)-X*P_import(1,t))/Unett;
      U_kunde(1,t)  = sqrt ( a(1,t) ^ 2 + b(1,t) ^ 2 );          
  end

%Spenningen i tilknytingspunktet kan bryte restriksjonen for utkobling.  
%Det kan vre tilfelle hvis ikke det er tilgjengelig "kt forbruk eller 
%lagring" eller restriksjon for aktiv regulering ikke er definert.
%Psol og Qsol settes til null for de periodene for spenningen 
%er over restriksjonen for utkobling. 
    if U_kunde(1,t) >= Vmax
      Psol(1,(t-1)*30) = 0;
      Qsol(1,(t-1)*30) = 0;

%Siden Psol og Qsol er oppgitt i minutt, m alle minutt innenfor halvtimen 
%settes til null i de perioder hvor restriksjonen for utkobling er brutt
      if Psol(1,(t-2)*30) == 0
          for i = ((t-2)*30):((t-1)*30)
             Psol(1,i) = 0;
             Qsol(1,i) = 0;
          end
      end
      
%Hvis lokal produksjon er null, s er import alltid lik forbruk, og derav
%endres importen.
      P_import(1,t)   = forbruk_effekt(t);

%Ny spenning i tilknytingspunktet regnes ut nr den lokale
%produksjonsenheten er koblet ut og forbruket forsynes av import
%Lik mte som i case 2
      if t == 1
        a(1,t) 		= ((Unett+sqrt(Unett^2-4*(R*P_import(1,t))...
                      +(X*Qsol(1,t))+(((R*Qsol(1,t)-...
                      X*P_import(1,t))/Unett)^2))))/2;
        b(1,t) 		= (R*Qsol(1,t)-X*P_import(1,t))/Unett;
        U_kunde(1,t)= sqrt ( a(1,t) ^ 2 + b(1,t) ^ 2 );
      else
        a(1,t) 		= ((Unett+sqrt(Unett^2-4*(R*P_import(1,t))...
                      +(X*Qsol(1,(t-1)*30))+(((R*Qsol(1,(t-1)*30)...
                      -X*P_import(1,t))/Unett)^2))))/2;
        b(1,t) 		= (R*Qsol(1,(t-1)*30)-X*P_import(1,t))/Unett;
        U_kunde(1,t)= sqrt ( a(1,t) ^ 2 + b(1,t) ^ 2 );
      end
    end
            
%sjekker om spenningsverdiene er lavere enn restriksjoene for produksjon 
%av reaktiv effekt og om produksjonsenheten er i drift i det tidspunktet
      elseif U_kunde(1,t) <= Vqned && t > 2 && Psol(1,(t-1)*30)>0

%Hvis spenningen er under, og produksjonenheten i drift, s defineres en 
%driftskurve, alts tan(arccos(effektfaktor) som tilsier produksjon av
%reaktiv effekt tilsvarende Effektfaktor = 0,95   
      driftskurven             = (0.32855);

%reaktiv effekt regnes ut basert p produksjon og driftskurve
%for de tidsperiodene hvor spenningen er lavere enn restriksjonen.
%Frst regnes det ut for halvtime, og deretter oppdateres alle minuttsdata          
        Qsol(1,(t-1)*30)= Psol(1,(t-1)*30) * driftskurven;
        if Qsol(1,(t-2)*30) > 0
            for i = ((t-2)*30):((t-1)*30)
                Qsol(1,i) = Psol(1,(t-1)*30) * driftskurven;
            end
        end
        
%Regnes ut en ny spenning i tilknytningspunktet for de perioder, hvor det
%produserer reaktiv effekt. Lik mte som i case 2
        if t == 1
            a(1,t) 		= ((Unett+sqrt(Unett^2-4*(R*P_import(1,t))...
                          +(X*Qsol(1,t))+(((R*Qsol(1,t)...
                          -X*P_import(1,t))/Unett)^2))))/2;
            b(1,t) 		= (R*Qsol(1,t)-X*P_import(1,t))/Unett;
            U_kunde(1,t)= sqrt ( a(1,t) ^ 2 + b(1,t) ^ 2 );
        else
            a(1,t) 		= ((Unett+sqrt(Unett^2-4*(R*P_import(1,t))...
                          +(X*Qsol(1,(t-1)*30))+(((R*Qsol(1,(t-1)*30)...
                          -X*P_import(1,t))/Unett)^2))))/2;
            b(1,t) 		= (R*Qsol(1,(t-1)*30)-X*P_import(1,t))/Unett;
            U_kunde(1,t)= sqrt ( a(1,t) ^ 2 + b(1,t) ^ 2 );
        end

%Hvis spenningen i tilknytingspunktet er lavere enn restriksjonen for
%utkobling s kobles produksjonen ut og Psol og Qsol settes til null for
%de periodene hvor spenningen er lavere enn restriksjonen. 
        if U_kunde(1,t) <= Vmin && t>2
            Psol(1,(t-1)*30) = 0;
            Qsol(1,(t-1)*30) = 0;

%Siden Psol og Qsol er oppgitt i minutt, m alle minutt innenfor 
%halvtimen settestil null i de perioder hvor restriksjonen for 
%utkobling er brutt                
            if Psol(1,(t-2)*30) == 0
                for i = ((t-2)*30):((t-1)*30)
                Psol(1,i) = 0;
                Qsol(1,i) = 0;
                end
            end
        end
%Hvis lokal produksjon er null, s er import alltid lik forbruk, og derav
%endres importen.             
        P_import(1,t)   = forbruk_effekt(t);
%Ny spenning i tilknytingspunktet utregens for perioder hvor
%produksjonenheten er utkoblet og import av effekt forsyner det lokale
%forbruket. Lik mte som i case 2     
        if t == 1
            a(1,t) 		= ((Unett+sqrt(Unett^2-4*(R*P_import(1,t))...
                          +(X*Qsol(1,t))+(((R*Qsol(1,t)-...
                          X*P_import(1,t))/Unett)^2))))/2;
            b(1,t) 		= (R*Qsol(1,t)-X*P_import(1,t))/Unett;
            U_kunde(1,t)= sqrt ( a(1,t) ^ 2 + b(1,t) ^ 2 );
        else
            a(1,t) 		= ((Unett+sqrt(Unett^2-4*(R*P_import(1,t))...
                          +(X*Qsol(1,(t-1)*30))+(((R*Qsol(1,(t-1)*30)...
                          -X*P_import(1,t))/Unett)^2))))/2;
            b(1,t) 		= (R*Qsol(1,(t-1)*30)-X*P_import(1,t))/Unett;
            U_kunde(1,t)= sqrt ( a(1,t) ^ 2 + b(1,t) ^ 2 );
        end
    end
end
%returner minimal import, som vil si maksimal eksport
    MinImport = min(P_import); 
    MinImport
end