%SIMULERINGSFIL FOR SAMTLIGE USE CASE


%Det er benyttet "..." tre punktum for  f plass til koden i oppgaven, 
%det betyr at steder hvor "..." er brukt fortsetter koden p neste
%linje

function Simulering
clear all
clc

valg = 99;
while valg ~= 0
%Input av variabler for simulering av 20.juni basert p forbruksdata 
%fra Demo Steinkjer, solcelleanlegget i oppgaven og 
%solstrling i Kristiansand.
R       = 0.641; %Nettets motstand (Ohm)
X       = 0.077; %Nettets reaktans (ohm)
A       = 41.5;  %Solcelleareal m^2
Eta     = 0.105; %Virkningsgrad for solcelleanlegget
Imax    = 1025.8;%Maksimal solstrling
Tmin    = 300;   %Soloppgang i minutter fra 00:00
Tmax    = 1320;  %Solnedgang i minutter fra 00:00
Delta_o = 38;    %Optimal helningsvinkel p det stedet
Delta_r = 38;    %Reell helningsvinkel til solcelleanlegget
Unett   = 230;   %Spenning i nettstasjon - eller annet referansepunkt
Tetta   = 0;     %Vinkelen til vekselretter-(arccos(effektfaktor))

%Definering av spenningsrestriksjoner for initiering av ulike tiltak
%for casene relatert tilknyting av et smart hus
Vmax    = 253;      %Maksimal spenning for utkobling
Vmin    = 207;      %Minimal spenning for utkobling
Vqopp   = 231;      %vre spenningsrestriksjon for reaktiv kompensering 
Vqned   = 229;      %Nedre spenningsrestriksjon for reaktiv kompensering 
Vlad    = 235;      %vre spenningsrestriksjon for kt forbruk/ladning 

%Forbruksdata fra Demo Steinkjer for 20. Juni
forbruk 		= importdata('Forbruk.txt'); %importerer forbruksdata
forbruk_tid 	= forbruk.data(:,1); %Tidspunkt for forbruk
forbruk_effekt  = forbruk.data(:,2); %Effektverdi per halvtime
str 			= size (forbruk_tid); %Strrelsen til forbruskmatrisen

%Matriser for solcelleanlegget
Isol 	= zeros (1,1441);%Initialiserer null-matriser for solstrling
Psol 	= zeros (1,1441);%Initialiserer null-matriser for Aktiv effekt
Qsol	= zeros (1,1441);%Initialiserer null-matriser for Reaktiv effekt

%Regner ut solstrlingen, Isol, som er antatt  ha en sinusform basert p 
%basert p soloppgang, nedgang og maks solstrling.Effektproduksjonen
%regnes ut basert p solcelleanleggets data og solstrlingen. 
%Reaktiv effekt regnes ut basert p aktiv effekt og tangens til 
%vinkelen som utgjr den aktuelle effektfaktoren,tan(arccos(effektfaktor)). 

for t = Tmin:Tmax %Kun verdier fra soloppgang til solnedgang
 Isol(1,t) = Imax * sin ( ( pi / ( Tmax - Tmin ) ) * (t - Tmin) );
 Psol(1,t) = Isol(1,t) * Eta * A * cos (((Delta_o - Delta_r)) * pi / 180);
 Qsol(1,t) = Psol(1,t) * tan ( Tetta );% i utgangspuntket = 0
end
    
%Matriser for det smarte huset
P_import    = zeros (1,str(1));%Effektimport
U_kunde     = zeros (1,str(1));%Spenningen i tilknytingspunkt
P_term      = zeros (1,str(1));%kt forbuk/lagring av aktiv effekt

%Regner ut og fyller opp matrisen for effektimport basert p
%differansen mellom forbruk og produksjon. Forbruk er gitt i halvtime, og 
%produksjon gitt i minutt, hvor effektimport kun bruker de sammnenlignede
%verdiene - alts halvtimesverdier og derav brukes "teller". 
teller = 1;
for t = 0:1440
    if t == 0
        P_import(1,teller)	 = forbruk_effekt(teller) - Psol(1,(t+1));
        teller				 = teller + 1;
    elseif t > 0 && t == forbruk_tid(teller)
        P_import(1,teller)	 = forbruk_effekt(teller) - Psol(1,(t));
        teller				 = teller + 1;
    end
end

%matriser for a og b, hvor a er det reelle og b det imaginre leddet i
%spenningen i tilknytingspunktet. Som omtalt i masteroppgaven i 
%tilknyting til use case for lokal spenningsregulering
%Strrelsen er gitt av strrelsen til matrisen for forbuket
a = zeros (1,str(1));
b = zeros (1,str(1));
    
%angir de mulige casene som kan simuleres. 
C1 = ['Case 1 - Visualisering'];
C2 = ['Case 2 - Spenning i tilknytningspunkt'];
C3 = ['Case 3 - Spenning i tilknytningspunkt med reaktiv kompensering'];
C4 = ['Case 4 - Spenning i tilknytningspunkt med aktiv effekt regulering'];
C5 = ['Case 5 - Spenningskarakteristikk i Mikronettstruktur'];
   
disp(C1);
disp(C2);
disp(C3);
disp(C4);
disp(C5);
   
%mulig  velge ulike case for simulering
valg = input('Velg case: ');

if valg == 2 
%Regner ut spenningen i tilknytningspunkt over dgnet for det smarte huset
%Dette er grunnscenarioet i use case "lokal spenningsregulering"
U_kunde = Case_2(Unett, P_import, Qsol, R, X, a, b, U_kunde, str);

elseif valg == 3
%Dette er scenario 2 og 3 i Use case "lokal spenningsregulering"
%med kompensering med reaktiv effekt, og eventuell utkobling. 
[Psol, U_kunde, P_import, Qsol] = Case_3(Unett, Psol, P_import, ...
Qsol, R, X, a, b, U_kunde, str, Vmax, Vmin, forbruk_effekt, Vqopp, Vqned);

elseif valg == 4
%Dette er scenario 2,3 og 4 i Use case " lokal spenningsregulering"
%Det vil si kompensering med reaktiv effekt, og regulering av 
%aktiv effekt innmating
[Psol, U_kunde, P_import, Qsol, P_term] = Case_4(Unett, Psol, ...
P_import, Qsol, R, X, a, b, U_kunde, str, Vmax, Vmin, forbruk_effekt, ...
Vqopp, Vqned, Vlad, P_term);

elseif valg == 5 
%Dette er simuleringen for "mikronett spenningsregulering".
%anvendes maksimal innmatet effekt fra et hus i case 2 som tilegnes 
%n antall hus. Det regnes ut spenningskarakterstikk i radialen med 
%"reaktiv kompensering", "trinnstyring" og regulering av aktiv effekt
%basert p formlene omtalt i masteroppgaven i kapitelen om use case
%Mikronett spenningsregulering.
[TotSpennFall_U, TotSpennFall_M, TotSpennFall_L, TotSpennFall_J, n]...
= Case_5(Unett, P_import, R, X);
end

%Skriver ut resultatene fra de ulike casene basert p hvilket case som
%er valgt. Hvor liten x er minuttsopplsning, og stor X er halvtime
x = 0:1:1440; %Isol, Psol, Qsol 
X = 0:30:1440; %Termisk/batteri lagring, P_import og spenning 

    if valg == 1 %Denne skriver ut Use case relatert Visualisering
        %solstrling over dagen
        subplot(2,2,1)
        plot(x,Isol);
        title('Solstrling');
        xlabel('Tid[min]');
        ylabel('Solstrling[W/m2]');
        
        %Produksjon over dagen
        subplot(2,2,2);
        plot(x,Psol);
        title('Lokal produksjon');
        xlabel('Tid[min]');
        ylabel('Aktiv effekt[W]');
        
        %Import av effekt - negativ = innmatning i overfringsnettet
        subplot(2,2,3);
        plot(X,P_import);
        title('Import av effekt');
        xlabel('Tid[min]');
        ylabel('Aktiv effekt[W]');
        
        %Fobruk av effekt hos hustanden
        subplot(2,2,4)
        plot(X,forbruk_effekt);
        title('Forbruk');
        xlabel('Tid[min]');
        ylabel('Aktiv effekt[W]');

    elseif any(valg == [2,3,4]) %Skriver ut "lokal spenningsregulering"
        %solstrling over dagen
        subplot(3,2,1);
        plot(x,Isol);
        title('Solstrling');
        xlabel('Tid[min]');
        ylabel('Solstrling[W/m2]');
        
        %Produksjon over dagen
        subplot(3,2,2);
        plot(x,Psol);
        title('Lokal produksjon');
        xlabel('Tid[min]');
        ylabel('Aktiv effekt[W]');
        
        %Reaktiv effekt forbruk/produksjon
        subplot(3,2,3);
        plot(x,Qsol);
        title('Reaktiv kompensering');
        xlabel('Tid[min]');
        ylabel('Reaktiv Effekt[VAr]');
        
        %Import av effekt - negativ = innmatning i overfringsnettet
        subplot(3,2,4);
        plot(X,P_import);
        title('Import av effekt');
        xlabel('Tid[min]');
        ylabel('Aktiv effekt[W]');
        
        %spenningen i tilknytingspunktet til det smarte huset
        subplot(3,2,5);
        plot(X,U_kunde);
        title('Spenning i tilknytningspunkt');
        xlabel('Tid[min]');
        ylabel('Spenning [V]');
        
        %Det kte forbruket eller lagringen, for aktiv effekt regulering
        subplot(3,2,6)
        plot(X,P_term);
        title('Termisk/Batteri lagring');
        xlabel('Tid[min]');
        ylabel('Aktiv Effekt [W]');

    %for mikronettstrukturen skrives det ut spenningskarakteristikk
     elseif valg == 5 
        x = 0:1:n; %antall hus som defineres
        
        %Karakterstikk for spenningen uten regulering
        subplot (4,1,1)
        plot(x,TotSpennFall_U);
        title('Spenningskarakteristikk');
        xlabel('Avstand(hus)');
        ylabel('Prosent[%]');
        
        %Karakterstikk for spenningen med reaktiv kompensering
        subplot(4,1,2);
        plot(x,TotSpennFall_M);
        title('Spenningskarakteristikk med reaktiv kompensering');
        xlabel('Avstand(hus)');
        ylabel('Prosent[%]');
        
        %Karakterstikk for spenningen med trinnstyring
        subplot(4,1,3);
        plot(x,TotSpennFall_J);
        title('Spenningskarakteristikk med Trinnstyring');
        xlabel('Avstand(hus)');
        ylabel('Prosent[%]');
        
        %Karakterstikk for spenningen med kt forbruk/lagring - aktiv
        %effekt regulering
        subplot(4,1,4);
        plot(x,TotSpennFall_L);
        title('Spenningskarakteristikk med kt forbruk/lagring');
        xlabel('Avstand(hus)');
        ylabel('Prosent[%]');
        
    end

end
    
end